use crate::fan::Fan;

use rppal::pwm::{Channel, Polarity, Pwm as Pwm_rpi};
use serde::{Deserialize, Serialize};

// See: https://serde/rs/remote-derive.html
#[derive(Serialize, Deserialize)]
#[serde(remote = "Channel")]
pub enum ChannelDef {
    Pwm0,
    Pwm1,
}

#[derive(Debug, Serialize, Deserialize)]
pub struct PwmConfig {
    max_temperature: f64,
    min_temperature: f64,
    frequency: f64,
    #[serde(with = "ChannelDef")]
    channel: Channel,
}

impl Default for PwmConfig {
    fn default() -> Self {
        PwmConfig {
            max_temperature: 70_f64,
            min_temperature: 60_f64,
            frequency: 0.1_f64,
            channel: Channel::Pwm0,
        }
    }
}

pub struct Pwm {
    max_temperature: f64,
    min_temperature: f64,
    pwm: Pwm_rpi,
}

impl Pwm {
    pub fn new(config: PwmConfig) -> Self {
        let pwm = Pwm_rpi::with_frequency(
            config.channel,
            config.frequency,
            0_f64,
            Polarity::Normal,
            false,
        )
        .unwrap();

        Pwm {
            max_temperature: config.max_temperature,
            min_temperature: config.min_temperature,
            pwm,
        }
    }
}

impl Fan for Pwm {
    fn update_temperature(&mut self, temperature: f64) {
        let percentage =
            (temperature - self.min_temperature) / (self.max_temperature - self.min_temperature);
        let percentage = if percentage > 1_f64 {
            1_f64
        } else {
            percentage
        };

        if temperature > self.min_temperature {
            self.pwm.enable().unwrap();
            self.pwm.set_duty_cycle(percentage).unwrap();
        } else {
            self.pwm.disable().unwrap();
        }
    }

    fn percentage(&self) -> f64 {
        self.pwm.duty_cycle().unwrap()
    }
}
